/*
 * ============================================================================
 *  Title:    Configuration Manager
 *  Author:   J. Zbiciak
 *  $Id: cfg.c,v 1.5 1999/09/17 13:10:01 im14u2c Exp $
 * ============================================================================
 *  This module manages the machine configuration.  It does commandline
 *  parsing and processes the configuration elements that were read in 
 *  via the config-file parser.
 *
 *  CFG owns the entire machine -- it is encapsulated in a cfg_t.
 * ============================================================================
 *  CFG_INIT     -- Parse command line and get started
 *  CFG_FILE     -- Parse a config file and extend the state of the machine.
 * ============================================================================
 */

static const char rcs_id[]="$Id: cfg.c,v 1.5 1999/09/17 13:10:01 im14u2c Exp $";

#include "../config.h"
#include "macros.h"
#include "periph/periph.h"
#include "cp1600/cp1600.h"
#include "mem/mem.h"
#include "pads/pads.h"
#include "gfx/gfx.h"
#include "snd/snd.h"
#include "stic/stic.h"
#include "ay8910/ay8910.h"
#include "ivoice/ivoice.h"
#include "debug/debug.h"
#include "speed/speed.h"
#include "event/event.h"
#include "cfg.h"


extern cp1600_t  cp1610;         /* CPU                                  */
extern stic_t    stic;           /* STIC                 0000-0032   16  */
extern ay8910_t  ay8914;         /* AY8914 Sound Chip    01F0-01FD    8  */
extern pad_t     pad0,pad1;      /* Controller pad 0, 1  01FE-01FF    8  */
extern debug_t   debug;          /* Debugger             0000-FFFF   16  */

extern gfx_t     gfx;            /* Graphics display                     */
extern snd_t     snd;            /* Sound mixer                          */
extern speed_t   speed;          /* Speed controller                     */

extern volatile int please_die;
extern volatile int reset;
#define XXXX

#ifdef macintosh
/* Macintosh requires prototypes */
void cfg_default(event_t *event);
#endif

/* ------------------------------------------------------------------------ */
/*  This, my dear friends, is the Intellivision.                            */
/* ------------------------------------------------------------------------ */
/*cfg_t intv;*/

/* ------------------------------------------------------------------------ */
/*  Default key bindings table.                                             */
/*                                                                          */
/*  Notes on mnemonics:                                                     */
/*      PD0L        Left controller pad on base unit.                       */
/*      PD0R        Right controller pad on base unit.                      */
/*      PD1L        Left controller pad on ECS unit.                        */
/*      PD1R        Right controller pad on ECS unit.                       */
/*      PDxx_KP     Left Key Pad                                            */
/*      PDxx_A      Left Action Button  ([T]op, [L]eft, [R]ight)            */
/*      PDxx_D      Right Disc                                              */
/*      N, NE, etc  Compass directions.                                     */
/* ------------------------------------------------------------------------ */

cfg_keyb_t  cfg_key_bind[] =
{
/* ------------------------------------------------------------------------ */
/*  Miscellaneous.                                                          */
/* ------------------------------------------------------------------------ */
{ "RESET",    "F10",      (v_uint_32*)&XXXX reset, { 0,   ~0U },{ 0,   ~0U } },
{ "QUIT",     "QUIT",     (v_uint_32*)&please_die, { ~0U, ~0U },{ ~0U, ~0U } },
{ "QUIT",     "ESCAPE",   (v_uint_32*)&please_die, { ~0U, ~0U },{ ~0U, ~0U } },
{ "QUIT",     "F12",      (v_uint_32*)&please_die, { ~0U, ~0U },{ ~0U, ~0U } },
{ "HIDE",     "HIDE",     (v_uint_32*)&gfx.hidden, { 0,   1   },{ 0,   1   } },
{ "HIDE",     "F1",       (v_uint_32*)&gfx.hidden, { 0,   1   },{ 0,   1   } },
 
/* ------------------------------------------------------------------------ */
/*  The numeric keypad.                                                     */
/* ------------------------------------------------------------------------ */
{ "PD0L_KP1", "KP7",      &XXXX pad0.l[1],  { 0,    ~0U  }, { 0,    0x81 } },
{ "PD0L_KP2", "KP8",      &XXXX pad0.l[2],  { 0,    ~0U  }, { 0,    0x41 } },
{ "PD0L_KP3", "KP9",      &XXXX pad0.l[3],  { 0,    ~0U  }, { 0,    0x21 } },
{ "PD0L_KP4", "KP4",      &XXXX pad0.l[4],  { 0,    ~0U  }, { 0,    0x82 } },
{ "PD0L_KP5", "KP5",      &XXXX pad0.l[5],  { 0,    ~0U  }, { 0,    0x42 } },
{ "PD0L_KP6", "KP6",      &XXXX pad0.l[6],  { 0,    ~0U  }, { 0,    0x22 } },
{ "PD0L_KP7", "KP1",      &XXXX pad0.l[7],  { 0,    ~0U  }, { 0,    0x84 } },
{ "PD0L_KP8", "KP2",      &XXXX pad0.l[8],  { 0,    ~0U  }, { 0,    0x44 } },
{ "PD0L_KP9", "KP3",      &XXXX pad0.l[9],  { 0,    ~0U  }, { 0,    0x24 } },
{ "PD0L_KPC", "KP0",      &XXXX pad0.l[10], { 0,    ~0U  }, { 0,    0x88 } },
{ "PD0L_KP0", "KP_PERIOD",&XXXX pad0.l[0],  { 0,    ~0U  }, { 0,    0x48 } },
{ "PD0L_KPE", "KP_ENTER", &XXXX pad0.l[11], { 0,    ~0U  }, { 0,    0x28 } },

/* ------------------------------------------------------------------------ */
/*  The number keys.                                                        */
/* ------------------------------------------------------------------------ */
{ "PD0R_KP1", "1",        &XXXX pad0.r[1],  { 0,    ~0U  }, { 0,    0x81 } },
{ "PD0R_KP2", "2",        &XXXX pad0.r[2],  { 0,    ~0U  }, { 0,    0x41 } },
{ "PD0R_KP3", "3",        &XXXX pad0.r[3],  { 0,    ~0U  }, { 0,    0x21 } },
{ "PD0R_KP4", "4",        &XXXX pad0.r[4],  { 0,    ~0U  }, { 0,    0x82 } },
{ "PD0R_KP5", "5",        &XXXX pad0.r[5],  { 0,    ~0U  }, { 0,    0x42 } },
{ "PD0R_KP6", "6",        &XXXX pad0.r[6],  { 0,    ~0U  }, { 0,    0x22 } },
{ "PD0R_KP7", "7",        &XXXX pad0.r[7],  { 0,    ~0U  }, { 0,    0x84 } },
{ "PD0R_KP8", "8",        &XXXX pad0.r[8],  { 0,    ~0U  }, { 0,    0x44 } },
{ "PD0R_KP9", "9",        &XXXX pad0.r[9],  { 0,    ~0U  }, { 0,    0x24 } },
{ "PD0R_KPC", "-",        &XXXX pad0.r[10], { 0,    ~0U  }, { 0,    0x88 } },
{ "PD0R_KP0", "0",        &XXXX pad0.r[0],  { 0,    ~0U  }, { 0,    0x48 } },
{ "PD0R_KPE", "=",        &XXXX pad0.r[11], { 0,    ~0U  }, { 0,    0x28 } },

/* ------------------------------------------------------------------------ */
/*  Action buttons.                                                         */
/* ------------------------------------------------------------------------ */
{ "PD0L_A_T", "RSHIFT",   &XXXX pad0.l[12], { 0,    ~0U  }, { 0,    0xA0 } },
{ "PD0L_A_L", "RALT",     &XXXX pad0.l[13], { 0,    ~0U  }, { 0,    0x60 } },
{ "PD0L_A_R", "RCTRL",    &XXXX pad0.l[14], { 0,    ~0U  }, { 0,    0xC0 } },

{ "PD0R_A_T", "LSHIFT",   &XXXX pad0.r[12], { 0,    ~0U  }, { 0,    0xA0 } },
{ "PD0R_A_L", "LALT",     &XXXX pad0.r[13], { 0,    ~0U  }, { 0,    0x60 } },
{ "PD0R_A_R", "LCTRL",    &XXXX pad0.r[14], { 0,    ~0U  }, { 0,    0xC0 } },

/* ------------------------------------------------------------------------ */
/*  Movement keys.                                                          */
/* ------------------------------------------------------------------------ */
{ "PD0L_D_E", "RIGHT",    &XXXX pad0.l[15], { ~1,   ~0U  }, { 0,    1  } },
{ "PD0L_D_N", "UP",       &XXXX pad0.l[15], { ~4,   ~0U  }, { 0,    4  } },
{ "PD0L_D_W", "LEFT",     &XXXX pad0.l[15], { ~16,  ~0U  }, { 0,    16 } },
{ "PD0L_D_S", "DOWN",     &XXXX pad0.l[15], { ~64,  ~0U  }, { 0,    64 } },

{ "PD0L_D_E", "K",        &XXXX pad0.l[15], { ~1,   ~0U  }, { 0,    1  } },
{ "PD0L_D_NE","O",        &XXXX pad0.l[15], { ~2,   ~0U  }, { 0,    2  } },
{ "PD0L_D_N", "I",        &XXXX pad0.l[15], { ~4,   ~0U  }, { 0,    4  } },
{ "PD0L_D_NW","U",        &XXXX pad0.l[15], { ~8,   ~0U  }, { 0,    8  } },
{ "PD0L_D_W", "J",        &XXXX pad0.l[15], { ~16,  ~0U  }, { 0,    16 } },
{ "PD0L_D_SW","N",        &XXXX pad0.l[15], { ~32,  ~0U  }, { 0,    32 } },
{ "PD0L_D_S", "M",        &XXXX pad0.l[15], { ~64,  ~0U  }, { 0,    64 } },
{ "PD0L_D_SE",",",        &XXXX pad0.l[15], { ~128, ~0U  }, { 0,    128} },

{ "PD0R_D_E", "D",        &XXXX pad0.r[15], { ~1,   ~0U  }, { 0,    1  } },
{ "PD0R_D_NE","R",        &XXXX pad0.r[15], { ~2,   ~0U  }, { 0,    2  } },
{ "PD0R_D_N", "E",        &XXXX pad0.r[15], { ~4,   ~0U  }, { 0,    4  } },
{ "PD0R_D_NW","W",        &XXXX pad0.r[15], { ~8,   ~0U  }, { 0,    8  } },
{ "PD0R_D_W", "S",        &XXXX pad0.r[15], { ~16,  ~0U  }, { 0,    16 } },
{ "PD0R_D_SW","Z",        &XXXX pad0.r[15], { ~32,  ~0U  }, { 0,    32 } },
{ "PD0R_D_S", "X",        &XXXX pad0.r[15], { ~64,  ~0U  }, { 0,    64 } },
{ "PD0R_D_SE","C",        &XXXX pad0.r[15], { ~128, ~0U  }, { 0,    128} },

};

/*
 * ============================================================================
 *  CFG_DEFAULT  -- Set all of the default settings for the Intellivision
 * ============================================================================
 */
void cfg_default(event_t *event)
{
    int i;

    for (i = 0; i < (int)(sizeof(cfg_key_bind)/sizeof(cfg_keyb_t)); i++)
    {
        event_map(event, cfg_key_bind[i].event, cfg_key_bind[i].word, 
                  cfg_key_bind[i].and_mask, cfg_key_bind[i].or_mask);
    }

}

#if 0
/*
 * ============================================================================
 *  CFG_INIT     -- Parse command line and get started
 * ============================================================================
 */
cfg_t *cfg_init(int argc, char * argv[])
{
    char * cfg_name;

    cfg_default(&intv);
}
#endif

/* ======================================================================== */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       */
/*  General Public License for more details.                                */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               */
/* ======================================================================== */
/*                 Copyright (c) 1998-1999, Joseph Zbiciak                  */
/* ======================================================================== */
